# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import AbsoluteTimeWindowClass
from .....schema_classes import CalendarIntervalClass
from .....schema_classes import DayOfWeekClass
from .....schema_classes import HolidayWindowClass
from .....schema_classes import PartitionSpecClass
from .....schema_classes import PartitionTypeClass
from .....schema_classes import TimeWindowClass
from .....schema_classes import TimeWindowSizeClass
from .....schema_classes import WeeklyWindowClass


AbsoluteTimeWindow = AbsoluteTimeWindowClass
CalendarInterval = CalendarIntervalClass
DayOfWeek = DayOfWeekClass
HolidayWindow = HolidayWindowClass
PartitionSpec = PartitionSpecClass
PartitionType = PartitionTypeClass
TimeWindow = TimeWindowClass
TimeWindowSize = TimeWindowSizeClass
WeeklyWindow = WeeklyWindowClass

# fmt: on
